<?php
/*
  Plugin Name: Expay Payment Gateway
  Plugin URI:
  Description: Allows you to use Expay gateway
  Version: 1.0.0
  Author: Expay Software Co., Ltd
  Text Domain: wc_expay_language
  Author URI: https://www.expay.asia
 */

/**
 * @param string $text
 * @return void
 */
function writeLog(string $text): void {
    file_put_contents('test.json', $text . "\n", FILE_APPEND);
}

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly
if (!defined('__DIR__')) {
    define('__DIR__', dirname(__FILE__));
}
include_once(__DIR__ . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'request_expay.php');

add_action('plugins_loaded', 'woocommerce_expay', 0);
add_action('plugins_loaded', 'load_expay_lang');


/**
 * @return void
 */
function load_expay_lang(): void
{
    $result = load_plugin_textdomain(
        'wc_expay_language',
        false,
        basename(dirname(__FILE__)) . DIRECTORY_SEPARATOR . 'languages' . DIRECTORY_SEPARATOR
    );
}

function woocommerce_expay()
{
    if (!class_exists('WC_Payment_Gateway')) {
        return;
    } // if the WC payment gateway class is not available, do nothing
    if (class_exists('wc_expay')) {
        return;
    }

    class wc_expay extends WC_Payment_Gateway
    {
        private static $error_file = 'errors_log_wc_expay.txt', $err_arr = [];
        private $callback_method;
        public $notify_url;

        /**
         * @param array $init_arr
         * @param string $mode
         * @param string $merchant
         * @param string $secret_key
         * @param string $info_order
         * @param string $info_item
         * @param string $debug
         */
        public function __construct(
            $init_arr = ['name' => 'Expay', 'id' => ''],
            $mode = '',
            $merchant = '',
            $secret_key = '',
            $info_order = 'no',
            $info_item = 'no',
            $debug = 'no'
        ) {
            $this->init_arr = $init_arr;
            $this->loger('init_arr', __line__);
            $this->loger($this->init_arr, __line__);
            $this->id = sanitize_file_name($init_arr ['name']);
            $this->icon = ((!empty($init_arr['image'])) && ($this->expay_check_image($init_arr['image']))) ? $init_arr['image'] : '';
            $this->has_fields = !empty($init_arr['attributes']);
            $this->title = $init_arr['name'];
            $this->method_title = $init_arr['name'];
            $this->notify_url = str_replace('https:', 'http:', add_query_arg('wc-api', 'wc_expay', home_url('/')));
            // Load the settings
            $this->init_form_fields();
            $this->init_settings();
            if (!(empty($mode) && empty($merchant) && empty($secret_key))) {
                $this->settings["mode"] = $mode;
                $this->settings["merchant"] = $merchant;
                $this->settings["secret_key"] = $secret_key;
                $this->settings["info_order"] = $info_order;
                $this->settings["info_item"] = $info_item;
                $this->settings["debug"] = $debug;
            }
            // Actions
            add_action('valid-expay-callback-request', [
                $this,
                'call_sdk_request'
            ]);
            add_action('woocommerce_receipt_' . $this->id, [
                $this,
                'receipt_page'
            ]);
            // Save options
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, [
                $this,
                'process_admin_options'
            ]);
            // Payment listener/API hook
            add_action('woocommerce_api_wc_expay', [
                $this,
                'check_callback_response'
            ], 10);

            if (!$this->is_valid_for_use()) {
                $this->enabled = false;
            }

        } // _construct


        /**
         * loger and debug function
         *
         * Add logs into file
         *
         * @access public
         * @param mixed $msg
         * @param int $line
         *
         */
        public function loger($msg, int $line = 0): void
        {
            if ($this->get_option('debug') === 'yes') {
                if($f_err = fopen(realpath(dirname(__FILE__)) . DIRECTORY_SEPARATOR . self::$error_file, 'a+')) {
                    fputs($f_err,
                        PHP_EOL . '/*** Loger ***/  Date:  ' . date('Y-m-d H:i:s') . ' [' . time() . ']' . PHP_EOL);
                    fputs($f_err, '[line: ' . $line . '] msg: ' . var_export($msg, true) . PHP_EOL);
                    fclose($f_err);
                }
            }

        }

        /**
         * add_action_links function.
         *
         * Adds action links inside the plugin overview
         *
         * @access public static
         * @param array $links
         * @return array
         */

        public static function add_action_links($links): array
        {
            $links = array_merge([
                '<a href="' . admin_url('admin.php?page=wc-settings&tab=checkout&section=wc_expay') . '">' . __('Settings',
                    'wc_expay_language') . '</a>',
                ],
                $links
            );
            array_push(
                $links,
                '<a href="https://expay.asia/"><img style="vertical-align:middle;" src="' . plugins_url() .
                DIRECTORY_SEPARATOR . plugin_basename(dirname(__FILE__)) .
                '/assets/images/expay.png" border="0" alt="Expay - easier way to pay online!" /></a>'
            );

            return $links;
        }

        /**
         * @return void
         */
        public function init_form_fields(): void
        {
            $this->form_fields = include('includes' . DIRECTORY_SEPARATOR . 'settings-expay.php');
        }

        /**
         * Validate the payment form
         * @return bool
         */
        public function validate_fields(): bool
        {
            /*(float)*/
            $sum = WC()->cart->total;
            $max_amount = (!empty($this->init_arr['max'])) ? $this->init_arr['max'] : '';
            $min_amount = (!empty($this->init_arr['min'])) ? $this->init_arr['min'] : '';
            $fix = (!empty($this->init_arr['commission']['fix'])) ? $this->init_arr['commission']['fix'] : '';
            $rate = (!empty($this->init_arr['commission']['rate'])) ? $this->init_arr['commission']['rate'] : '';

            if (round($sum, 2) < round((($sum * $rate) + $fix), 2)) {
                wc_add_notice(__('Wrong payment amount (Out of range) ', 'wc_expay_language'), 'error');

                return false;
            }
            if (round($sum, 2) < round($min_amount, 2)) {
                wc_add_notice(__('Wrong payment amount (< min) ', 'wc_expay_language'), 'error');

                return false;
            }
            if (round(WC()->cart->total, 2) > round((float)$max_amount, 2)) {
                wc_add_notice(__('Wrong payment amount (> max) ', 'wc_expay_language'), 'error');

                return false;
            }
            foreach ($this->init_arr['attributes'] as $attribute) {
                $regexp = $attribute['regexp'];
                if (!empty($regexp) && ($attribute['required'] === '1')) {
                    if (preg_match('/' . $regexp . '/', $_POST['key_' . $attribute['key'] . '_' . $this->init_arr['id']])) {
                        WC()->session->set('key_' . $attribute['key'] . '_' . $this->init_arr['id'],
                            $_POST['key_' . $attribute['key'] . '_' . $this->init_arr['id']]);
                    } else {
                        wc_add_notice(__('Field does not match mask (!=regexp) ', 'wc_expay_language') . $attribute['name'],
                            'error');

                        return false;
                    }
                } else {
                    if (empty($_POST['key_' . $attribute['key'] . '_' . $this->init_arr['id']]) && $attribute['required'] === '1') {
                        wc_add_notice(
                            __('Empty required field (!=required) ',
                                'wc_expay_language') . $attribute['name'],
                            'error'
                        );

                        return false;
                    } else {
                        WC()->session->set(
                            'key_' . $attribute['key'] . '_' . $this->init_arr['id'],
                            $_POST['key_' . $attribute['key'] . '_' . $this->init_arr['id']]
                        );
                    }
                }
            }
            WC()->session->set('getMethod', $this->init_arr);

            return true;
        }

        /**
         * There are no payment fields for sprypay, but we want to show the description if set.
         *
         * @return void
         **/
        public function payment_fields(): void
        {
            if (!empty($this->init_arr['attributes'])) {
                echo '<div style="
     				background-color:PapayaWhip;
     				padding: 5px;
     				border: solid 1px black;
     				position: relative;
     				margin: auto; ">';

                foreach ($this->init_arr['attributes'] as $attribute) {
                    echo '<b>' . $this->v['name'] . '</b> </p>[' . $attribute['description'] . ']</p>';
                    echo '<input type="text" id="key_' . $attribute['key'] . '_' . $this->init_arr['id'] . '" name="key_'
                        . $attribute['key'] . '_' . $this->init_arr['id'] . '" autocomplete="off" class="input_text" value="">'
                        . (($attribute['required']) ? '<font color="red">*</font>' : null) . '</p>';
                }

                echo "</div>";
            }

        }

        /**
         * Check if this gateway is enabled and available in the user's country
         * @return bool
         */
        public function is_valid_for_use(): bool
        {
            $currency = ['AUD', 'BRL', 'CAD', 'MXN', 'NZD', 'HKD', 'SGD', 'USD', 'EUR', 'JPY', 'TRY', 'NOK', 'CZK', 'DKK',
                'HUF', 'ILS', 'MYR', 'PHP', 'PLN', 'SEK', 'CHF', 'TWD', 'THB', 'GBP', 'RMB', 'RUB'];

            if (!in_array(get_woocommerce_currency(), apply_filters('woocommerce_expay_supported_currencies', $currency))) {
                return false;
            }

            return true;
        }

        /** Check available icon
         *
         */
        public function expay_check_image($url): bool
        {
            return true;
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_HEADER, true);
            curl_setopt($ch, CURLOPT_NOBODY, true);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
            curl_setopt($ch, CURLOPT_TIMEOUT, 10);
            curl_setopt($ch, CURLOPT_URL, $url);
            $out = curl_exec($ch);
            $status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);

            return ($status == 200) ? true : false;
        }

        /**
         * Admin Panel Options
         * - Options for bits like 'title' and availability on a country-by-country basis
         *
         * @return void
         * @since 0.1
         **/
        public function admin_options(): void
        {
            wp_enqueue_style(
                'my_expay_admin_css',
                plugins_url(basename(plugin_dir_path(__FILE__))) . '/css/style.css',
                false,
                false
            );

            echo include('includes' . DIRECTORY_SEPARATOR . 'admin_options.php');
        }

// End admin_options()

        /**
         * @param $order
         * @return array
         */
        public function get_expay_args($order): array
        {
            $billing = [
                "BILL_FNAME" => $order->billing_first_name,
                "BILL_LNAME" => $order->billing_last_name,
                "BILL_ADDRESS" => $order->billing_address_1,
                "BILL_ADDRESS2" => $order->billing_address_2,
                "BILL_CITY" => $order->billing_city,
                "BILL_PHONE" => $order->billing_phone,
                "BILL_EMAIL" => $order->billing_email,
                "BILL_COUNTRYCODE" => $order->billing_country,
                "BILL_ZIPCODE" => $order->billing_postcode,
                "ORDER_SHIPPING" => number_format(
                    $order->get_total_shipping() + $order->get_shipping_tax(),
                    2,
                    '.',
                    ''
                ),
                "PRICES_CURRENCY" => get_woocommerce_currency(),
                "ORDER_REF" => $order->id
            ];
            // Shipping
            $delivery = [
                "DELIVERY_FNAME" => $order->shipping_first_name,
                "DELIVERY_LNAME" => $order->shipping_last_name,
                "DELIVERY_ADDRESS" => $order->shipping_address_1,
                "DELIVERY_ADDRESS2" => $order->shipping_address_2,
                "DELIVERY_CITY" => $order->shipping_city,
                "DELIVERY_PHONE" => $order->billing_phone,
                "DELIVERY_EMAIL" => $order->billing_email,
                "DELIVERY_COUNTRYCODE" => $order->shipping_country,
                "DELIVERY_ZIPCODE" => $order->shipping_postcode,
            ];
            $orderArray = array_merge($billing, $delivery);
            if (count($order->get_items()) > 0) {
                foreach ($order->get_items() as $item) {
                    $orderArray['ORDER_PNAME'][] = $item['name']; # Array with data of goods
                    $orderArray['ORDER_QTY'][] = $item['qty']; # Array with data of counts of each goods
                    $orderArray['ORDER_PRICE'][] = $order->get_item_subtotal($item, false); #number_format( $order->get_total() - $order->get_shipping() - $order->get_shipping_tax() + $order->get_order_discount(), 2, '.', '' ); # Array with prices of goods
                    $orderArray['ORDER_PCODE'][] = $item['product_id']; #"testgoods_".$item['id']; # Array with codes of goods
                    $orderArray['ORDER_VAT'][] = $this->get_option("VAT");# Array with VAT of each goods  => from settings
                }
            }
            if ($order->get_total_discount()) {
                $orderArray['DISCOUNT'] = $order->get_total_discount();
            }
            $expay_args['Expay_data'] = $orderArray;
            $expay_args = apply_filters('woocommerce_expay_args', $expay_args);

            return $expay_args;
        }

        /**
         * @param WC_Order $order
         * @param string $method
         * @param array $posted
         * @return bool|string|void
         */
        public function expay_send(WC_Order $order, string $method = '', array $posted = [])
        {
            $expayRequest = null;

            try {
                $expayRequest = new \expay\request_expay(
                    $this->get_option("mode"),
                    $this->get_option("merchant"),
                    htmlspecialchars_decode($this->get_option("secret_key"))
                );
                $this->loger($expayRequest, __line__);
            } catch (expay\ExpayException $e) {
                $this->loger('expay_send error:' . $e->getMessage(), __line__);
                wc_add_notice(__('Error in proccess payment. Please try again later.', 'wc_expay_language'), 'error');

                return;
            }
            if ($expayRequest) {
                $arr = [];
                if ($method === 'initPayment') {
                    $arr = [
                        'service_id' => $this->init_arr['id'],
                        'amount' => number_format($order->get_total(), 2, '.', ''),
                        'order' => $order->get_order_number(),
                        '_successUrl' => $order->get_view_order_url() . '?&status=success',
                        '_waitingUrl' => $order->get_view_order_url() . '?&status=waiting',
                        '_rejectUrl' => $order->get_view_order_url() . '?&status=reject',

                    ];

                    foreach ($this->init_arr['attributes'] as $attribute) {
                        $tmp_arr[$attribute['key']] = $posted['key_' . $attribute['key'] . '_' . $this->init_arr['id']];
                    }

                    if ($tmp_arr) {
                        $arr = array_merge($arr, $tmp_arr);
                    }

                } elseif ($method === 'getStatus') {
                    $arr = [
                        'order' => $order->get_order_number()
                    ];
                }
                try {
                    $this->loger('send data array', __line__);
                    $this->loger($arr, __line__);
                    $post_fields = $expayRequest->$method($arr);

                } catch (expay\ExpayException $e) {
                }
                if ($post_fields) {
                    $out = $post_fields;
                } else {
                    self::$err_arr = \expay\request_expay::$err_a['error'];
                    $out = false;
                }

                return $out;
            }

            $this->loger('expay_send error', __line__);
            wc_add_notice(__('Error in proccess payment. Please try again later.', 'wc_expay_language'), 'error');

            return;
        }

        /**
         * @param $order_id
         * @return array
         */
        public function process_payment($order_id): array
        {
            global $woocommerce;

            $order = wc_get_order($order_id);
            $this->loger('Order:' . $order_id, __line__);
            $this->loger('getMethod:', __line__);
            $this->loger(WC()->session->get('getMethod', ''), __line__);
            update_post_meta(
                $order->id,
                date('Y-m-d H:i:s') . '_getMethod:',
                var_export(WC()->session->get('getMethod', ''), true)
            );
            $out = $this->expay_send($order, 'initPayment', $_POST);
            $this->loger('initPayment', __line__);
            $this->loger($out, __line__);
            if($out) {
                update_post_meta(
                    $order->id,
                    date('Y-m-d H:i:s') . '_initPayment:',
                    var_export($out, true)
                );
            } else {
                update_post_meta(
                    $order->id,
                    date('Y-m-d H:i:s') . '_initPayment:',
                    var_export(self::$err_arr, true)
                );
            }

            if ((is_array($out)) && ($out['status'] === 206)) {
                update_post_meta($order_id, 'initPayment', $out);
                $woocommerce->cart->empty_cart();
                $order->update_status('pending', __('Awaiting payment', 'wc_expay_language'));
                $order->add_order_note(__('Awaiting payment', 'wc_expay_language'));
                $showData = '';
                $redirectUrl = '';
                foreach ($out['attributes'] as $attribute) {
                    if ($attribute['key'] == 'redirectUrl') {
                        $redirectUrl = $attribute['value'];
                    }
                    if ($attribute['key'] == 'showData') {
                        $showData = base64_decode($attribute['value']);
                        $description = $attribute['description'];
                    }
                }
                WC()->session->set('redirectUrl', $redirectUrl);
                WC()->session->set('showData', $showData);
                WC()->session->set('description', $description);

                if (!empty(WC()->session->get('showData', ''))) {
                    $showDataUrl = str_replace(
                        '<a href="',
                        '',
                        str_replace(
                            '">Download receipt</a>',
                            '',
                            $showData
                        )
                    );
                    WC()->session->set('showDataurl', $showDataUrl);
                }
                return [
                    'result' => 'success',
                    'redirect' => $order->get_checkout_payment_url(true)
                ];
            } elseif ((!$out) && (isset(self::$err_arr['code'])) && (isset(self::$err_arr['message']))) {
                update_post_meta($order_id, '_initPayment_error', self::$err_arr);
                $this->loger(
                    'Order:' . $order_id . ' Error:' . '[Code ' . self::$err_arr['code'] . ']' . self::$err_arr['message'],
                    __line__
                );
                $order->add_order_note('Order:' . $order_id . ' Error:' . '[Code ' . self::$err_arr['code'] . ']' . self::$err_arr['message']);
                $order->update_status(
                    'cancelled',
                    'Order:' . $order_id . ' Error:' . '[Code ' . self::$err_arr['code'] . ']' . self::$err_arr['message']
                );
                wc_add_notice(__('Error in proccess payment. Please try again later.', 'wc_expay_language'),
                    'error');
                return [
                    'result' => 'success',
                    'redirect' => $order->get_cancel_order_url()
                ];
            } elseif ((is_array($out)) && (isset($out['status']))) {
                $this->loger('Order:' . $order_id, __line__);
                $this->loger('Order was rejected.', __line__);
                $order->add_order_note(__('Order was rejected.', 'wc_expay_language'));
                $order->update_status('cancelled', __('Order was rejected', 'wc_expay_language'));
                wc_add_notice(__('Error in proccess payment. Please try again later.', 'wc_expay_language'),
                    'error');
                return [
                    'result' => 'success',
                    'redirect' => $order->get_cancel_order_url()
                ];
            } else {
                wc_add_notice(__('Error in proccess payment. Please try again later.', 'wc_expay_language'),
                    'error');
            }
            $order->add_order_note(__('Order was rejected.',
                    'wc_expay_language') . 'Order:' . $order_id . ' Error:' . '[Code ' . self::$err_arr['code'] . ']' . self::$err_arr['message']);
            $order->update_status('cancelled', __('Order was rejected',
                    'wc_expay_language') . 'Order:' . $order_id . ' Error:' . '[Code ' . self::$err_arr['code'] . ']' . self::$err_arr['message']);
            $this->loger('Order:' . $order_id . ' Error:' . '[Code ' . self::$err_arr['code'] . ']' . self::$err_arr['message'],
                __line__);

            return [
                'result' => 'success',
                'redirect' => $order->get_cancel_order_url()
            ];
        }

        /**
         * @param $order_id
         * @return string
         */
        public function generate_expay_form($order_id): string
        {
            $order = new WC_Order($order_id);
            $redirectUrl = WC()->session->get('redirectUrl', '');
            $showData = WC()->session->get('showData', '');
            $showDataUrl = WC()->session->get('showDataurl', '');
            $description = WC()->session->get('description', '');
            if (!empty($redirectUrl)) {
                if ($description) {
                    $info = '<p>' . $description . '</p>';
                } else {
                    $info = '';
                }
                $out_form = '<a class="button alt" id="receipt_' . $this->id . '_button"  href="' . $redirectUrl . '">' . __('Pay via',
                        'wc_expay_language') . ' ' . $this->init_arr['name'] . '</a>' . ' ' .
                    ' <a class="button cancel" id="cancel_' . $this->id . '_button" href="' . esc_url($order->get_cancel_order_url()) . '">' . __('Cancel and return in cart',
                        'wc_expay_language') . '</a>';
            }
            if (!empty($showData)) {
                if ($description) {
                    $info = '<p>' . $description . '</p>';
                } else {
                    $info = '';
                }
                if ($showData !== $showDataUrl) {
                    $info .= '<br><h4>' . __('Attention! Save the document to pay before a page will be closed!',
                            'wc_expay_language') . '</h4>';
                    $out_form = $showData . '</br>' . $info . '</br><a class="button alt" id="view_order_' . $this->id . '_button"  href="' . $order->get_view_order_url() . '">' . __('View order ',
                            'wc_expay_language') . ' ' . $this->init_arr['name'] . '</a>';
                    $cancel = '<a class="button cancel" id="cancel_' . $this->id . '_button" href="' . esc_url($order->get_cancel_order_url()) . '">' . __('Cancel and return to cart',
                            'wc_expay_language') . '</a>';
                    $out_form = $out_form . '&nbsp' . $cancel;
                } else {
                    $out_form = $showData . '<br>' . $info . '<h4>' . __('Attention! Save the document to pay before a page will be closed!',
                            'wc_expay_language') . '</h4></br><a class="button alt" id="receipt_' . $this->id . '_button"  href="' . $order->get_view_order_url() . '">' . __('View order ',
                            'wc_expay_language') . ' ' . $this->init_arr['name'] . '</a></br></br><a class="button cancel" id="cancel_' . $this->id . '_button" href="' . esc_url($order->get_cancel_order_url()) . '">' . __('Cancel and return to cart',
                            'wc_expay_language') . '</a>';
                }
            }

            wc_enqueue_js('
			jQuery("#cancel_' . $this->id . '_button").click( function(){
				jQuery("body").block
				({
					message: "' . esc_js(__('Now you will be redirect in cart!', 'wc_expay_language')) . '",
					baseZ: 99999,
					overlayCSS:
					{
						background: "#fff",
						opacity: 0.6
					},
	            centerY: 0,
                css: {
                		top:  ($(window).height() - 50) /2 + \'px\',
                		left: ($(window).width() - 400) /2 + \'px\',
                		width: \'400px\',
                		padding:        "10px",
				        zindex:         "9999999",
				        textAlign:      "center",
				        color:          "#999",
				        border:         "5px solid #aaa",
				        backgroundColor:"#fff",
				        cursor:         "wait",
				        lineHeight:		"36px",}
				});
				jQuery(".blockOverlay").attr("title","Click on any place on form to unblock").click($.unblockUI);

			});
		');
            wc_enqueue_js('
			jQuery("#submit_' . $this->id . '_submit_button").click( function(){
				jQuery("body").block({
					timeout:30000,
					message: "' . esc_js(($redirectUrl <> '') ? (__('Thank you for your order. Now you will be redirected to the system',
                        'wc_expay_language') . ' ' . $this->init_arr['name'] . ' ' . __('for pay order.',
                        'wc_expay_language')) : (__('Thank you for your order. Save a document',
                        'wc_expay_language') . ' ' . $this->init_arr['name'] . ' ' . __('for pay order.',
                        'wc_expay_language'))) . '",
					baseZ: 99999,
					fadeIn: 700,
            		fadeOut: 700,
					overlayCSS:
					{
						background: "#fff",
						opacity: 0.6
					},
	            centerY: 0,
                css: {
                		top:  ($(window).height() - 50) /2 + \'px\',
                		left: ($(window).width() - 400) /2 + \'px\',
                		width: \'400px\',
                		padding:        "10px",
				        zindex:         "9999999",
				        textAlign:      "center",
				        color:          "#999",
				        border:         "5px solid #aaa",
				        backgroundColor:"#fff",
				        cursor:         "wait",
				        lineHeight:		"36px",}
				});
				jQuery(".blockOverlay").attr("title","Click on any place on form to unblock").click($.unblockUI);
			});
		');

            return $out_form;
        }

        /**
         * receipt_page
         **/
        public function receipt_page($order_id)
        {
            $order = new WC_Order($order_id);
            if (WC()->session->get('redirectUrl', '') <> '') {
                echo '<p>' . __('Thank you for your order, please press pay button to payment.',
                        'wc_expay_language') . '</p>';
                echo $this->generate_expay_form($order);
            } else {
                if (WC()->session->get('showData', '') <> '') {
                    echo '<p>' . __('Thank you for your order. Save a document to payment.',
                            'wc_expay_language') . '</p>';
                    echo $this->generate_expay_form($order);
                } else {
                    echo '<p>' . __('Error in proccess payment. Please try again later.',
                            'wc_expay_language') . '</p><br><a class="button cancel" id="cancel_' . $this->id . '_button" href="' . esc_url($order->get_cancel_order_url()) . '">' . __('Cancel and return to cart',
                            'wc_expay_language') . '</a>';
                    return;
                }
            }
            if ($this->get_option('info_order') == 'yes') {
                woocommerce_order_details_table($order_id);
            }
            if ($this->get_option('info_item') == 'yes') {
                $order_items = '';
                $expay_args = $this->get_expay_args($order);
                /**/
                foreach ($expay_args['Expay_data'] as $k => $v) {
                    if ($k == 'ORDER_PNAME') {
                        $order_items = '<table><th>' . __('Information about goods', 'wc_expay_language') . ':</th>';
                        foreach ($v as $kk => $vv) {
                            $order_items .= '<tr><td>' . $vv . '<b> x ' . $expay_args['Expay_data']['ORDER_QTY'][$kk] . ' x ' . $expay_args['Expay_data']['ORDER_PRICE'][$kk] . ' ' . $expay_args['Expay_data']['PRICES_CURRENCY'] . '</b></td><td>' . round(((float)$expay_args['Expay_data']['ORDER_QTY'][$kk] * (float)$expay_args['Expay_data']['ORDER_PRICE'][$kk]),
                                    2) . ' ' . $expay_args['Expay_data']['PRICES_CURRENCY'] . '</td></tr>';
                        }
                        $order_items .= '</table>';
                    }


                }
                echo $order_items;
            }
        }

        /**
         * @param string $posted
         * @param string $method
         * @return mixed
         */
        public function check_order($posted = '', $method = '')
        {
            global $woocommerce;

            $this->loger($posted, __line__);
            $this->loger($method, __line__);
            if (!empty($posted) && isset($posted['order']) && ('check' == $method || 'pay' == $method || 'status' == $method)) {
                $order = new WC_order($posted['order']);
                update_post_meta($order->id, date('Y-m-d H:i:s') . '_' . $method . ':', var_export($posted, true));
                $status = $order->get_status();
                $final_status = array(
                    'failed',
                    'completed',
                    'cancelled',
                    'refunded',
                    $this->get_option('po_status')
                );
                if (
                    (!empty($initpayment)) && (in_array($status,
                        $final_status)) && ($order->get_total() == $posted['amount']) && ($initpayment['id'] == $posted['id']) && ($initpayment['service']['id'] == $posted['service_id']) && ($initpayment['order'] == $posted['order']) && ((time() - 86400) < $posted['timestamp']) && ((time() + 86400) > $posted['timestamp'])
                    ||
                    (('status' == $method) && ($initpayment['order'] == $posted['order']) && ((time() - 86400) < $posted['timestamp']) && ((time() + 86400) > $posted['timestamp']))
                ) {
                    $info = 'Expay order:' . $order->id . ' ID:' . $posted['id'] . ' is a final status';
                    $this->loger($info, __line__);
                    $order->add_order_note = __('Expay order:',
                            'wc_expay_language') . $order->id . ' ID:' . $posted['id'] . __(' is a final status',
                            'wc_expay_language');

                    return $info;
                } else {
                    if (
                        (!empty($initpayment)) && ($order->get_total() == $posted['amount']) && ($initpayment['id'] == $posted['id']) && ($initpayment['service']['id'] == $posted['service_id']) && ($initpayment['order'] == $posted['order']) && ((time() - 86400) < $posted['timestamp']) && ((time() + 86400) > $posted['timestamp'])
                        ||
                        (('status' == $method) && ($initpayment['order'] == $posted['order']) && ((time() - 86400) < $posted['timestamp']) && ((time() + 86400) > $posted['timestamp']))
                    ) {
                        $this->loger($order->get_total() . ' = ' . $posted['amount'] . __(' Validation complete:',
                                'wc_expay_language'), __line__);
                        $order->add_order_note = __('Expay ID:', 'wc_expay_language') . $posted['id'];
                        if ('pay' == $method) {
                            $this->update_order($this->get_option('po_status'),
                                __('Order was paid', 'wc_expay_language'), $order, $posted, $method);
                            $order->add_order_note(__('Expay: Order payment completed success', 'wc_expay_language'));
                            // Reduce stock levels
                            ('yes' == $this->get_option('stock_reduce')) ? $order->reduce_order_stock() : null;
                        }

                        return false;
                    } else {
                        //(('check' == $method) ? ($err = 475) : ($err = 474));
                        $err = 475;
                        self::$err_arr['code'] = $err;
                        $info = 'Expay error validation. Order:' . $order->id . ' ID:' . $posted['id'] . ' code:' . $err . ' wrong amount, timestamp, order, or initPayment meta';
                        $this->loger($info . ' : ' . $order->get_total() . ' != ' . $posted['amount'], __line__);
                        self::$err_arr['message'] = $info;
                        $order->add_order_note = __('Expay error validation. Order:',
                                'wc_expay_language') . $order->id . ' ID:' . $posted['id'] . ' code:' . $err . __(' wrong amount, timestamp, order, or initPayment meta',
                                'wc_expay_language'); //'Expay ID:' . $posted['id'];
                        //update_post_meta($order->id, __('Expay order info','wc_expay_language'), var_export($posted, true));

                        if (!in_array($status, $final_status)) {
                            $order->update_status('failed', sprintf(__('Error validation ',
                                    'wc_expay_language') . $method . ' code: ' . $err . ' : ' . __('wrong order parameters',
                                    'wc_expay_language') . ' (ID Order: %s, ID Expay: %s, amount Expay : %s ).',
                                $posted['order'], $posted['id'], $posted['amount']));
                        }

                        return $info;
                    }
                }

                $info = __FUNCTION__ . ': Post is wrong';
                $this->loger($info, __line__);
                self::$err_arr['code'] = 500;
                self::$err_arr['message'] = $info;

                return $info;
            }
        }

        public function update_order($status, $msg, $order, $posted, $method)
        {
            $order->update_status($status,
                sprintf($msg . ' (ID Order: %s, ID Expay: %s) status: ' . $posted['status'], $posted['order'],
                    $posted['id']));
            update_post_meta($order->id, date('Y-m-d H:i:s') . '_' . $method . ':', var_export($posted, true));

            return;
        }

        /**
         * @param array $posted
         * @param string $method
         * @return bool|mixed|string
         */
        public function setStatus_order(array $posted = [], string $method = '')
        {
            $this->loger('setStatus_order', __line__);
            $this->loger($posted, __line__);

            if (!empty($posted) && isset($posted['status']) && isset($posted['order']) && isset($posted['id'])) {


                $order = new WC_order($posted['order']);
                $this->loger($order->status, __line__);
                // Check order if already completed
                $final_status = array(
                    'failed',
                    'completed',
                    'cancelled',
                    'refunded',
                    $this->get_option('po_status')
                );
                if (in_array($order->status, $final_status)) {
                    $this->loger('Order finished', __line__);
                    $this->update_order($order->status, __('Order finished', 'wc_expay_language'), $order, $posted,
                        $method);

                    return $order->status;
                } else {
                    if (in_array($posted['status'], array(201, 203))) {
                        $this->loger('Order on-hold', __line__);
                        $this->update_order('on-hold', __('Order on-hold', 'wc_expay_language'), $order, $posted,
                            $method);

                        return 'on-hold';
                    } else {
                        if (in_array($posted['status'], array(206))) {
                            $this->loger('Order pending', __line__);
                            $this->update_order('pending', __('Order pending', 'wc_expay_language'), $order, $posted,
                                $method);

                            return 'pending';
                        } else {
                            if (in_array($posted['status'], array(204))) {
                                $this->loger('Order cancelled', __line__);
                                $this->update_order('cancelled', __('Order cancelled', 'wc_expay_language'), $order,
                                    $posted, $method);

                                return 'cancelled';
                            } else {
                                if ($posted['status'] == (207)) {
                                    $this->loger('Order refunded', __line__);
                                    $this->update_order('refunded', __('Order refunded', 'wc_expay_language'), $order,
                                        $posted, $method);

                                    //
                                    return 'refunded';
                                } else {
                                    if ($posted['status'] == (205)) {
                                        $this->loger('Order completed', __line__);
                                        $this->update_order($this->get_option('po_status'),
                                            __('Order was paid', 'wc_expay_language'), $order, $posted, $method);
                                        $order->add_order_note(__('Expay: Order payment completed success',
                                            'wc_expay_language'));
                                        return $this->get_option('po_status');
                                    } else {
                                        $this->loger('Order another status', __line__);
                                        $this->update_order('failed', __('Order another status', 'wc_expay_language'),
                                            $order, $posted, $method);
                                        $order->add_order_note(__('Order another status', 'wc_expay_language'));

                                        return 'failed';
                                    }
                                }
                            }
                        }
                    }
                }
            }

            return false;
        }

        /**
         * @param array $posted
         * @return bool|string
         * @throws \expay\ExpayException
         */
        public function validate_request(array $posted = [])
        {
            $this->loger('validate_request', __line__);
            try {
                $f = new \expay\my_expay($this->get_option("mode"), $this->get_option("merchant"),
                    htmlspecialchars_decode($this->get_option("secret_key")));
            } catch (ExpayException $e) {
                $this->loger($e->getMessage(), __line__);
            }
            if (!is_null($f->call_class)) {
                try {
                    $post_fields = $f->parse_post($posted);
                    $this->loger($post_fields, __line__);
                    if ($post_fields) //(!is_array($post_fields))
                    {
                        return sprintf('%s', $post_fields);
                    }
                } catch (ExpayException $e) {
                }
            }
            $this->loger('getinfoexpayerror', __line__);
            $post_fields = $f->getinfoexpayerror();
            $tmp_a = array('status' => $post_fields['code'], 'message' => $post_fields['message']);
            $tmp = $f->response_expay($tmp_a);
            print $tmp;

            return false;
        }

        /**
         * @param array $posted
         * @throws \expay\ExpayException
         */
        public function call_sdk_request(array $posted = [])
        {
            $this->loger($posted, __line__);
            if ($posted == '') {
                return;
            }

            $post_fields = $this->validate_request($posted);
            if (!$post_fields) {
                return;
            }

            $this->callback_method = $post_fields;
            $tmp_a = array('status' => '401', 'message' => 'Invalid default method ');
            $order = null;
            try {
                $order = new WC_order(absint(sprintf('%d', $posted['order'])));
            } catch(Exception $e) {
            }

            if (isset($posted['id']) && $order->id <> 0 && isset($posted['order']) && $order->id == $posted['order'] && $order->get_total() == $posted['amount'] && ((time() - 86400) < $posted['timestamp']) && ((time() + 86400) > $posted['timestamp'])) {
                if (('check' == $post_fields) && !empty($posted['id']) && !empty($posted['order']) && !empty($posted['timestamp']) && !empty($posted['service_id']) && !empty($posted['amount'])) {
                    $status = $order->get_status();
                    if (!in_array(self::$err_arr['code'], array(475, 500))) {
                        switch ($status) {
                            case $this->get_option('po_status'):
                                $tmp_a = array('status' => 270, 'message' => 'completed');
                                break;
                            case 'pending':
                                $tmp_a = array('status' => 270, 'message' => 'pending');
                                break;
                            case 'failed':
                                $tmp_a = array('status' => 475, 'message' => 'failed');
                                break;
                            case 'processing':
                                $tmp_a = array('status' => 270, 'message' => 'processing');
                                break;
                            case 'completed':
                                $tmp_a = array('status' => 270, 'message' => 'completed ');
                                break;
                            case 'on-hold':
                                $tmp_a = array('status' => 475, 'message' => 'on-hold');
                                break;
                            case 'cancelled':
                                $tmp_a = array('status' => 475, 'message' => 'cancelled');
                                break;
                            case 'refunded':
                                $tmp_a = array('status' => 475, 'message' => 'refunded');
                                break;
                        }
                    } else {
                        $tmp_a = array('status' => self::$err_arr['code'], 'message' => self::$err_arr['message']);
                    }
                    $this->loger($tmp_a, __line__);
                } else {
                    if (('pay' == $post_fields) && !empty($posted['id']) && !empty($posted['order']) && !empty($posted['timestamp']) && !empty($posted['service_id']) && !empty($posted['amount'])) {
                        $status = $order->get_status();
                        if (!in_array(self::$err_arr['code'], array(474, 500))) {
                            if ($status == 'pending') {
                                $this->loger($order->get_total() . ' = ' . $posted['amount'] . __(' Validation complete:',
                                        'wc_expay_language'), __line__);
                                $order->add_order_note = __('Expay ID:', 'wc_expay_language') . $posted['id'];
                                $this->update_order($this->get_option('po_status'),
                                    __('Order was paid', 'wc_expay_language'), $order, $posted, 'pay');
                                $order->add_order_note(__('Expay: Order payment completed success', 'wc_expay_language'));
                                // Reduce stock levels
                                ('yes' == $this->get_option('stock_reduce')) ? $order->reduce_order_stock() : null;
                                $tmp_a = array('status' => 205, 'message' => 'completed ');
                            } else {
                                switch ($status) {
                                    case $this->get_option('po_status'):
                                        $tmp_a = array('status' => 205, 'message' => 'completed ');
                                        break;
                                    case 'failed':
                                        $tmp_a = array('status' => 475, 'message' => 'failed');
                                        break;
                                    case 'processing':
                                        $tmp_a = array('status' => 205, 'message' => 'processing');
                                        break;
                                    case 'completed':
                                        $tmp_a = array('status' => 205, 'message' => 'completed ');
                                        break;
                                    case 'on-hold':
                                        $tmp_a = array('status' => 475, 'message' => 'on-hold');
                                        break;
                                    case 'cancelled':
                                        $tmp_a = array('status' => 475, 'message' => 'cancelled');
                                        break;
                                    case 'refunded':
                                        $tmp_a = array('status' => 475, 'message' => 'refunded');
                                        break;
                                }
                            }
                        } // end (self::$err_arr['code']<>500)
                        else {
                            $tmp_a = array(
                                'status' => self::$err_arr['code'],
                                'message' => self::$err_arr['message']
                            );
                        }
                        $this->loger($tmp_a, __line__);
                    } else {
                        if (($post_fields == 'status') && !empty($posted['order']) && !empty($posted['timestamp'])) {
                            $status = $order->get_status();
                            $this->loger($status, __line__);
                            if (!in_array(self::$err_arr['code'], array(474, 500))) {
                                switch ($status) {
                                    case $this->get_option('po_status'):
                                        $tmp_a = array('status' => '205', 'message' => 'completed ');
                                        break;
                                    case 'pending':
                                        $tmp_a = array('status' => '206', 'message' => 'pending');
                                        break;
                                    case 'failed':
                                        $tmp_a = array('status' => '201', 'message' => 'failed');
                                        break;
                                    case 'processing':
                                        $tmp_a = array('status' => '206', 'message' => 'processing');
                                        break;
                                    case 'completed':
                                        $tmp_a = array('status' => '205', 'message' => 'completed ');
                                        break;
                                    case 'on-hold':
                                        $tmp_a = array('status' => '206', 'message' => 'on-hold');
                                        break;
                                    case 'cancelled':
                                        $tmp_a = array('status' => '204', 'message' => 'cancelled');
                                        break;
                                    case 'refunded':
                                        $tmp_a = array('status' => '207', 'message' => 'refunded');
                                        break;
                                }
                                //(self::$err_arr['code']=='474')? $tmp_a['status']=self::$err_arr['code']:null;
                            } // end (self::$err_arr['code']<>500)
                            else {
                                $tmp_a = array(
                                    'status' => self::$err_arr['code'],
                                    'message' => self::$err_arr['message']
                                );
                            }
                            $this->loger($tmp_a, __line__);
                        } else {
                            $this->loger('Error 401 Invalid requested order parameters', __line__);
                            $tmp_a = array('status' => '401', 'message' => 'Invalid requested order parameters');
                            $this->loger($tmp_a, __line__);
                        }
                    }
                }

            } else {
                self::$err_arr['code'] = 475;
                self::$err_arr['message'] = 'Payment with the specified parameters is not found: [ Order:' . $posted['order'] . ', ID: ' . $posted['id'] . ', amount: ' . $posted['amount'] . ' ]';
                $tmp_a = array('status' => self::$err_arr['code'], 'message' => self::$err_arr['message']);
                $this->loger($tmp_a, __line__);
            }
            try {
                $f = new \expay\my_expay($this->get_option("mode"), $this->get_option("merchant"),
                    htmlspecialchars_decode($this->get_option("secret_key"))); //''
                if (!is_null($f->call_class)) {
                    $tmp = $f->response_expay($tmp_a);
                    if ($tmp) {
                        echo $tmp;
                        die();
                    }
                }
            } catch (ExpayException $e) {
                // handle exception
                //обработка ошибки, если она нужна, вывод соббщения, логического кода и т.п.
                //$err=$e->getMessage();
                //echo $err;
            }

            $post_fields = $f->getinfoexpayerror();
            $tmp_a = array('status' => $post_fields['code'], 'message' => $post_fields['message']);
            $tmp = $f->response_expay($tmp_a);
            echo $tmp;
            die();
        }

        public function prepare_expay_answer($tmp_a)
        {
            try {
                $f = new \expay\my_expay($this->get_option("mode"), $this->get_option("merchant"),
                    htmlspecialchars_decode($this->get_option("secret_key")));
                if (!is_null($f->call_class)) {
                    $tmp = $f->response_expay($tmp_a);
                    if ($tmp) {
                        return $tmp;
                    }
                }
            } catch (ExpayException $e) {
                // handle exception
                //обработка ошибки, если она нужна, вывод соббщения, логического кода и т.п.
                //$err=$e->getMessage();
                //echo $err;
            }
        }

        private function multiexplode($delimiters, $string)
        {

            $ready = str_replace($delimiters, $delimiters[0], $string);
            $launch = explode($delimiters[0], $ready);

            return $launch;
        }

        public function check_callback_response()
        {
            global $woocommerce;
            $ip_address = $_SERVER['REMOTE_ADDR'];
            $this->loger('$ip_address', __line__);
            $this->loger($ip_address, __line__);
            $this->loger('$_REQUEST', __line__);
            $this->loger($_REQUEST, __line__);
            $ips = $this->get_option("ip_address");
            header('Content-Type: application/json');
            if (!empty($ips)) {
                $ips = $this->multiexplode(array(",", ", "), $this->get_option("ip_address"));
                $this->loger('$ips', __line__);
                $this->loger($ips, __line__);
                $this->loger('$_SERVER', __line__);
                $this->loger($_SERVER, __line__);
                if ((array_search($ip_address, $ips) === false) && (!isset($_GET['order'])) && (!isset($_GET['pid']))) {
                    $this->loger('Error! Wrong IP adress:', __line__);
                    $this->loger($_SERVER["REMOTE_ADDR"], __line__);
                    header('HTTP/1.1 403 Forbidden');
                    exit;
                }
            }

            if (empty($_POST) && !empty($_GET['status']) && (isset($_GET['order'])) && (isset($_GET['pid']))) {
                $order_id = absint(sprintf('%d', $_GET['order']));
                $p_id = absint(sprintf('%d', $_GET['pid']));
                $order = new WC_Order($order_id);
                $initpayment = get_post_meta($order->id, 'initPayment');
                $initpayment = array_shift($initpayment);
                if (($p_id == $initpayment['id']) && (in_array(sprintf('%s', $_GET['status']), array(
                        'success',
                        'waiting',
                        'reject'
                    )))
                ) {
                    if (isset($_GET['status']) && isset($_GET['pid']) && isset($_GET['order'])) {

                        if ($this->get_option('mode') && $this->get_option('merchant') && $this->get_option('secret_key')) {
                            $posted['status'] = sprintf('%s', $_GET['status']);
                            $posted['order'] = absint(sprintf('%d', $_GET['order']));
                            $posted['id'] = absint(sprintf('%d', $_GET['pid']));
                            $this->loger('redirect', __line__);
                            $this->loger($posted, __line__);

                            if ('success' == sprintf('%s', $_GET['status'])) {
                                $this->update_order($this->get_option('po_status'),
                                    __('Order was paid', 'wc_expay_language'), $order, $posted, 'redirect[success]');
                            }
                            if ('reject' == sprintf('%s', $_GET['status'])) {

                                $this->update_order('cancelled', __('Order cancelled', 'wc_expay_language'), $order,
                                    $posted, 'redirect[reject]');
                            }
                            if ('waiting' == sprintf('%s', $_GET['status'])) {
                                $this->update_order('pending', __('Order pending', 'wc_expay_language'), $order,
                                    $posted, 'redirect[waiting]');
                            }
                        }
                    }

                    $post_fields = $this->expay_send($order, 'getStatus');
                    if ($post_fields) {
                        $woocommerce->cart->empty_cart();
                        $this->setStatus_order($post_fields, 'getStatus(redirectUrl)');
                    }
                    wp_redirect($order->get_view_order_url());
                } else {
                    $tmp_a = array(
                        'status' => '500',
                        'message' => sprintf("In order:%s pid:%s not found", $order_id, $p_id)
                    );
                    $tmp = $this->prepare_expay_answer($tmp_a);
                    $this->loger('Error! Order & pid not found!', __line__);
                    $this->loger(sprintf("In order:%s pid:%s not found", $order_id, $p_id), __line__);
                    wp_die(sprintf("In order:%s pid:%s not found <p>[ %s ]", $order_id, $p_id, $tmp));
                }
            } else if (!empty($_POST) && empty($_GET['status'])) {
                if (!$this->validate_request(stripslashes_deep($_POST))) {
                    exit;
                }
                // Optional: kill all other output buffering
                while (ob_get_level() > 0) {
                    ob_end_clean();
                }
                ob_start();
                $_POST = stripslashes_deep($_POST);
                do_action("valid-expay-callback-request", $_POST);
                // Ignore connection-closing by the client/user
                ignore_user_abort(true);
                // Get your output and send it to the client
                $content = ob_get_contents();         // Get the content of the output buffer
                ob_end_clean();                      // Close current output buffer
                $len = strlen($content);             // Get the length
                header('Connection: close');         // Tell the client to close connection
                header("Content-Length: $len");     // Close connection after $size character
                echo $content;                       // Output content
                flush();                             // Force php-output-cache to flush to browser.
                // See caveats below.
                // Optional: kill all other output buffering
                while (ob_get_level() > 0) {
                    ob_end_clean();
                }
            } else {
                $tmp_a = array('status' => '500', 'message' => 'EXPAY callback Request Failure!');
                $tmp = $this->prepare_expay_answer($tmp_a);
                wp_die("EXPAY callback Request Failure <p>[ $tmp ]");
            }
        }

        public function getstatus_expay_order()
        {
            $this->loger('getstatus_expay_order', __line__);
            $this->loger($_POST, __line__);
            $_POST = stripslashes_deep($_POST);
            sleep(5);
            $this->loger('after timer', __line__);
            if (('status' == @sprintf('%s',
                        $_POST['method'])) && (isset($_POST['order'])) && ($this->validate_request($_POST))) {
                $this->loger('status:', __line__);
                $order = new WC_Order($_POST['order']);
                $post_fields = $this->validate_request($_POST);
                if ('status' == $post_fields) {
                    $post_fields = $this->expay_send($order, 'getStatus');
                }
                if (!$post_fields) {
                    return;
                }
                $this->loger('getStatus expay answer:', __line__);
                $this->loger($post_fields, __line__);
                $this->setStatus_order($post_fields, 'getStatus(status)');
            }
            exit;
        }

        public function my_request($mode, $merchant, $secret_key, $method, $order_id = '')
        {
            try {
                $out = null;
                $tmp = new \expay\request_expay($mode, $merchant, htmlspecialchars_decode($secret_key));
            } catch (expay\Exception $e) {
                $this->loger('Error:', __line__);
                $this->loger($e->getMessage(), __line__);
                return false;
            }
            $arr = '';
            if ('getStatus' == $method) {
                $arr = array(
                    'order' => $order_id
                );
            }
            $post_fields = null;
            try {
                $post_fields = $tmp->$method($arr);
            } catch (Exception $e) {
                $this->loger('Error:', __line__);
                $this->loger($e->getMessage(), __line__);
            }
            if (!($out = $post_fields)) {
                $err_arr = $tmp->getinfoexpayerror();
                throw new expay\ExpayException('Expay\'s Response was empty. [' . $err_arr['code'] . ' ' . $err_arr['message'] . '' . $err_arr['timestamp'] . ']',
                    $err_arr['code']);
            }
            return $out;
        }
    } // end class wc_expay
    /**
     * Add the gateway to WooCommerce
     **/
    function my_admin_error_notice()
    {
        $err_class = "error";
        $message = __('Empty fields mode or merchant or secret_key in gateway',
                'wc_expay_language') . ' Expay: ' . '<a href="' . admin_url('admin.php?page=wc-settings&tab=checkout&section=wc_expay') . '">' . __('Settings',
                'wc_expay_language') . '</a>';
        echo "<div class=\"$err_class\"> <p>$message</p></div>";
    }

    function add_expay_gateway($methods, array $expay_methods = [])
    {
        $base_obj_gw = new wc_expay;
        if (empty(WC()->session)) {
            $methods[] = $base_obj_gw;
            if (
                (is_admin()) &&
                ((isset($_REQUEST['woocommerce_Expay_mode'])) || (isset($_REQUEST['woocommerce_Expay_merchant'])) || (isset($_REQUEST['woocommerce_Expay_secret_key'])))
            ) {
                if ((empty($_REQUEST['woocommerce_Expay_mode'])) || (empty($_REQUEST['woocommerce_Expay_merchant'])) || (empty($_REQUEST['woocommerce_Expay_secret_key']))) {
                    add_action('admin_notices', 'my_admin_error_notice');
                }
            } else {
                if (
                !($base_obj_gw->get_option('mode') && $base_obj_gw->get_option('merchant') && $base_obj_gw->get_option('secret_key'))
                ) {
                    add_action('admin_notices', 'my_admin_error_notice');
                }
            }
        } else {
            if (
            ('yes' == $base_obj_gw->get_option('enabled'))
            ) {
                if ($base_obj_gw->get_option('mode') && $base_obj_gw->get_option('merchant') && $base_obj_gw->get_option('secret_key')
                    && (absint(get_query_var('order-pay')) == 0)
                ) {
                    if (
                        !is_checkout()
                        && WC()->cart->needs_payment()
                    ) {
                        $expay_methods = WC()->session->get('expay_methods', null);
                    } else {
                        try {
                            $expay_methods = call_user_func_array(
                                array($base_obj_gw, "my_request"),
                                array(
                                    $base_obj_gw->get_option('mode'),
                                    $base_obj_gw->get_option('merchant'),
                                    $base_obj_gw->get_option('secret_key'),
                                    'getMethods'
                                )
                            );

                            WC()->session->set('expay_methods', $expay_methods);


                        } catch (expay\ExpayException $e) {
                            WC()->session->set('expay_methods', null);
                            $base_obj_gw->loger('Error getMethods:', __line__);
                            $base_obj_gw->loger($e->getMessage(), __line__);
                            (is_woocommerce() || is_checkout()) ? (wc_add_notice(__('Some payments gateway are unavailable now.',
                                'wc_expay_language'), 'notice')) : null;//$e->getMessage()
                        }
                    }
                } else {
                    if (
                        (WC()->session->get('getMethod', '') <> '')
                        && (absint(get_query_var('order-pay')) > 0)
                    ) {
                        $expay_methods['methods'][0] = WC()->session->get('getMethod', '');
                        $methods[] = new wc_expay(WC()->session->get('getMethod', ''),
                            $base_obj_gw->get_option('mode'),
                            $base_obj_gw->get_option('merchant'),
                            $base_obj_gw->get_option('secret_key'),
                            $base_obj_gw->get_option('info_order'),
                            $base_obj_gw->get_option('info_item'),
                            $base_obj_gw->get_option('debug')
                        );
                        return $methods;
                    } else {
                        if (!($base_obj_gw->get_option('mode') && $base_obj_gw->get_option('merchant') && $base_obj_gw->get_option('secret_key'))) {
                            (is_woocommerce() || is_checkout()) ? (wc_add_notice(__('Some payments gateway are unavailable now.',
                                'wc_expay_language'), 'notice')) : null;
                            (is_admin()) ? add_action('admin_notices', 'my_admin_error_notice') : null;
                            return $methods;
                        }
                    }
                }
                if (!empty($expay_methods)) { //echo '<hr>step 0<br>';
                    if (array_key_exists('methods', $expay_methods)) { //echo '<hr>step 1<br>';
                        foreach ($expay_methods['methods'] as $k => $v) {
                            if (is_array($v) && (@isset(WC()->cart->total)) && (in_array($v['type'], array(
                                    'online',
                                    'offline'
                                )))
                            ) {
                                $methods[] = new wc_expay($v,
                                    $base_obj_gw->get_option('mode'),
                                    $base_obj_gw->get_option('merchant'),
                                    $base_obj_gw->get_option('secret_key'),
                                    $base_obj_gw->get_option('info_order'),
                                    $base_obj_gw->get_option('info_item'),
                                    $base_obj_gw->get_option('debug')
                                );
                            }
                        }
                    }
                }
            }
        }
        return $methods;
    }

    class update_status_expay_orders extends wc_expay
    {
        public static $count_start = 0, $view_order = 0, $show_data, $initpayment;
        public static $err_s_arr = array(), $succ_s_arr = array();

        public function __construct($page = '')
        {
            if (self::$count_start == 0)
            {
                self::$count_start++;
            }
        }


        public function view_order_action()
        {
            global $woocommerce, $wp;
            if (isset($_GET['pid']) && isset($_GET['order'])) {
                $woocommerce->cart->empty_cart();
            }

            $order_id = absint($wp->query_vars['view-order']);
            $order = new WC_Order($order_id);
            self::$initpayment = get_post_meta($order->id, 'initPayment');
            self::$initpayment = array_shift(self::$initpayment);
            if ($order_id == self::$initpayment['order']) {
                $status = $order->get_status();


                if (isset($_GET['status']) && isset($_GET['pid']) && isset($_GET['order'])) {
                    $obj = new wc_expay();
                    if ($obj->get_option('mode') && $obj->get_option('merchant') && $obj->get_option('secret_key')) {
                        $posted['status'] = sprintf('%s', $_GET['status']);
                        $posted['order'] = absint(sprintf('%d', $_GET['order']));
                        $posted['id'] = absint(sprintf('%d', $_GET['pid']));
                        $obj->loger('redirect', __line__);
                        $obj->loger($posted, __line__);
                        $order = new WC_order($posted['order']);
                        if ('success' == sprintf('%s', $_GET['status'])) {
                            $obj->update_order($obj->get_option('po_status'), __('Order was paid', 'wc_expay_language'),
                                $order, $posted, 'redirect[success]');
                        }
                        if ('reject' == sprintf('%s', $_GET['status'])) {

                            $obj->update_order('cancelled', __('Order cancelled', 'wc_expay_language'), $order, $posted,
                                'redirect[reject]');
                        }
                        if ('waiting' == sprintf('%s', $_GET['status'])) {
                            $obj->update_order('pending', __('Order pending', 'wc_expay_language'), $order, $posted,
                                'redirect[waiting]');
                        }
                    }
                }


                $this->update_status_orders($order_id, $status);
            }
        } //end

        //admin panel and my-account

        public function status_orders_action()
        {
            global $woocommerce;
            if (isset($_GET['pid']) && isset($_GET['order'])) {
                $woocommerce->cart->empty_cart();
            }
            $init_arr = array(
                'numberposts' => -1,
                'post_type' => array(0 => 'shop_order', 1 => 'shop_order_refund',),
                'post_status' => array(0 => 'wc-pending',) //1 => 'wc-on-hold',2 => 'wc-processing',
            );
            if (!is_admin()) {
                $init_arr['meta_key'] = '_customer_user';
                $init_arr['meta_value'] = get_current_user_id();
            }
            (is_admin()) ? add_action('admin_notices', array($this, 'settings_expay_in_orders')) : null;
            $customer_orders = get_posts($init_arr);
            if ($customer_orders) {

                foreach ($customer_orders as $customer_order) {
                    $order = new WC_Order();
                    $order->populate($customer_order);
                    $order_id = $order->get_order_number();
                    $initpayment = get_post_meta($order->id, 'initPayment');
                    $initpayment = array_shift($initpayment);
                    if ($order_id == $initpayment['order']) {
                        #функция которую запускаем после проверок
                        $this->update_status_orders($order_id, $order->get_status());

                    }
                }
            }
        }// end

        public function update_status_orders($order_id = '', $order_status = '')
        {
            if (($order_id <> '') && (in_array($order_status, array('pending')))) //'on-hold'
            {
                $obj = new wc_expay();
                if ($obj->get_option('mode') && $obj->get_option('merchant') && $obj->get_option('secret_key')) {
                    try {
                        $status = $obj->my_request(
                            $obj->get_option('mode'),
                            $obj->get_option('merchant'),
                            $obj->get_option('secret_key'),
                            'getStatus',
                            $order_id
                        );
                    } catch (expay\ExpayException $e) {
                        $obj->loger('Error update_status_orders:' . $order_id, __line__);
                        $obj->loger($e->getMessage(), __line__);
                        $str = __('Error in proccess update status of order # ',
                                'wc_expay_language') . $order_id . ' [' . __('State of order has not been Updated!',
                                'wc_expay_language') . ']';

                        self::$err_s_arr[$order_id] = $str;
                        (is_view_order_page() || is_account_page()) ? (wc_add_notice($str,
                            'error')) : null;

                        (is_admin()) ? add_action('admin_notices', array(
                            $this,
                            'error_update_expay_orders_notice'
                        )) : null;
                        return false;
                    }
                    if (!$status) {
                        return false;
                    }
                    $result = $obj->setStatus_order($status, 'getStatus(view)');
                    if ($result) {
                        $str = __('Updated status of order # ',
                                'wc_expay_language') . $order_id . '[ ' . $order_status . '  >  ' . $result . ' ]';
                        self::$succ_s_arr[$order_id] = $str;
                        (is_view_order_page() || is_account_page()) ? (wc_add_notice($str,
                            'notice')) : null;
                        (is_admin()) ? add_action('admin_notices', array(
                            $this,
                            'success_update_expay_orders_notice'
                        )) : null;
                        return true;
                    }
                }  // end if ($obj->get_option('mode')...
                else {
                    (is_admin()) ? add_action('admin_notices', 'my_admin_error_notice') : null;
                    return;
                }
            }
        } // end function update_status_orders

        public function success_update_expay_orders_notice()
        {
            $t = new wc_expay;
            $err_class = "notice";
            $err = '';
            foreach (self::$succ_s_arr as $k => $v) {
                ($err) ? $err .= ', ' : null;
                $err .= '<a href="' . get_edit_post_link($k) . '">' . $k . '</a>';

            }
            if ($t->get_option('list_succ_order') === 'yes') {
                $message = '<b>' . __('Updated statuses of orders from gateway',
                        'wc_expay_language') . ' Expay:</b>' . $err;
                echo '<div class="' . $err_class . '"> <p style="background-color: SpringGreen;"> ' . $message . '</p></div>';
            }

        }

        public function error_update_expay_orders_notice()
        {
            $t = new wc_expay;
            $err_class = "error";
            $err = '';
            foreach (self::$err_s_arr as $k => $v) {
                ($err) ? $err .= ', ' : null;
                $err .= '<a href="' . get_edit_post_link($k) . '">' . $k . '</a>';

            }

            if ('yes' == $t->get_option('list_unsucc_order')) {
                $message = '<b>' . __('During update statuses of orders from gateway',
                        'wc_expay_language') . ' Expay ' . __('has been errors', 'wc_expay_language') . ':</b>' . $err;
                echo '<div class="' . $err_class . '"> <p style="background-color: Salmon;"> ' . $message . '</p></div>';
            }

        }

        public function settings_expay_in_orders()
        {
            $message = '<b>' . __('Quick redirect to gateway', 'wc_expay_language') . ' Expay ' . __('settings: ',
                    'wc_expay_language') . '</b>';
            echo '<div class="notice"> <p style="background-color: #c0c0c0;">' . $message . '<a href="' . admin_url('admin.php?page=wc-settings&tab=checkout&section=wc_expay') . '">' . __('Expay gateway Settings',
                    'wc_expay_language') . '</a></p></div>';
        }
    }// end class update_status_expay_orders

    /**
     * @param string $str
     */
    function action_function(string $str = ''): void
    {
        $checkObj = new wc_expay;
        if ((update_status_expay_orders::$count_start === 0) && ($checkObj->get_option('enabled') === 'yes')) {
            $a = new update_status_expay_orders;
            ('view-order' === $str) ? $a->view_order_action() : $a->status_orders_action();
        }
    }

    function view_initpayment_showdata()
    {
        if (!empty(update_status_expay_orders::$initpayment)) {
            $showData = '';
            $description = '';
            foreach (update_status_expay_orders::$initpayment['attributes'] as $attribute) {
                if ($attribute['key'] === 'showData') {
                    $showData = isset($attribute['value']) ? base64_decode($attribute['value']) : '';
                    $description = isset($attribute['description']) ? $attribute['description'] : '';
                }
            }
            echo $description . '<br />' . $showData;
        }
    }


    function shop_order_load()
    {
        global $pagenow, $typenow;
        if (('All Posts' === get_admin_page_title()) && ('edit.php' === $pagenow) && ('shop_order' === $typenow)) {
            action_function();
        }

    }

    add_action('manage_shop_order_posts_custom_column', 'MY_COLUMNS_VALUES_FUNCTION', 2);
    function MY_COLUMNS_VALUES_FUNCTION($column)
    {
        global $post;

        //start editing, I was saving my fields for the orders as custom post meta
        //if you did the same, follow this code
        if ($column == 'EXPAY_COLUMN_STATUS') {
            $order = new WC_Order();
            $order->populate($post);
            $order_id = $order->get_order_number();
            $initpayment = get_post_meta($order->id, 'initPayment');
            $initpayment = array_shift($initpayment);
            $initPayment_error = get_post_meta($order->id, '_initPayment_error');
            $initPayment_error = array_shift($initPayment_error);
            if ($initpayment) {
                //if order in initpayment
                if ($order_id == $initpayment['order']) {
                    // status was not updated
                    if (array_key_exists($order_id, update_status_expay_orders::$err_s_arr)) {
                        echo 'not updated';
                    } // status was updated
                    else {
                        if
                        (array_key_exists($order_id, update_status_expay_orders::$succ_s_arr)) {
                            echo 'updated';
                        } // final status, status was not updated
                        else {
                            echo 'final';
                        }
                    }
                }
                //
            } else {
                if ($initPayment_error) {
                    echo $initPayment_error['code'];
                }
            }
        }

    }

    add_filter('manage_edit-shop_order_columns', 'EXPAY_COLUMNS_FUNCTION', 11);
    function EXPAY_COLUMNS_FUNCTION($columns)
    {
        $new_columns = (is_array($columns)) ? $columns : array();
        unset($new_columns['order_expay_actions']);

        $new_columns['EXPAY_COLUMN_STATUS'] = 'Expay Up';
        return $new_columns;
    }

    # update status if open my-account page
    # update status if open view-order page
    add_action('admin_menu', 'shop_order_load'); // 722ms
    add_action('woocommerce_view_order', 'view_initpayment_showdata');
    add_filter('woocommerce_payment_gateways', 'add_expay_gateway');
    if (class_exists('update_status_wc_expay_orders')) {
        return;
    }
    add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'wc_expay::add_action_links');

}

require_once 'api/routes.php';

/**
 * Get list of all products
 *
 * @param WP_REST_Request $request
 * @return mixed
 */
function apiGetProductsList(WP_REST_Request $request)
{
    try {
        $params = $request->get_params();
        $attributes = [
            'include',
            'exclude',
            'parent',
            'parent_exclude',
            'limit',
            'page',
            'paginate',
            'offset',
            'order',
            'orderby',
            'return',
            'sku',
            'tag',
            'category',
            'weight',
            'length',
            'width',
            'height',
            'price',
            'regular_price',
            'sale_price',
            'total_sales',
            'virtual',
            'downloadable',
            'featured',
            'sold_individually',
            'manage_stock',
            'reviews_allowed',
            'backorders',
            'visibility',
            'stock_quantity',
            'stock_status',
            'tax_status',
            'tax_class',
            'tax_class',
            'download_limit',
            'download_expiry',
            'average_rating',
            'review_count',
            'date_created',
            'date_modified',
            'date_on_sale_from',
            'date_on_sale_to',
            'name',
        ];

        $dynamicParameters = [];
        foreach ($params as $key => $attribute) {
            if (in_array($key, $attributes)) {
                if (is_array($attribute)) {
                    foreach ($attribute as $subKey => $value) {
                        $dynamicParameters[$key][] = $value;
                    }
                } else {
                    $dynamicParameters[$key] = $attribute;
                }
            } elseif ($key === 'search') {
                $dynamicParameters['s'] = $attribute;
            }
        }

        $defaultParameters = [
            'orderby' => 'name',
            'order' => 'ASC',
            'return' => 'objects',
        ];

        $queryParameters = array_merge($dynamicParameters, $defaultParameters);
        $queryParameters['status'] = 'publish';
        $queryParameters['type'] = 'simple';

        $query = new WC_Product_Query($queryParameters);

        if (!$products = $query->get_products()) {
            return new WP_Error('not found', 'No product with the given name', ['status' => 404]);
        }

        $productsArray = array_map(function(WC_Product_Simple $product) {
            return $product->get_data();
        }, $products);

        echo json_encode($productsArray);
    } catch (Exception $e) {
        throw new $e;
    }
    exit();
}

/**
 * Get product details
 *
 * @param WP_REST_Request $request
 * @return mixed
 */
function apiGetProductDetails(WP_REST_Request $request)
{
    try {
        $productId = $request->get_param('id');
        /** @var WC_Product_Simple $product */
        if (!$product = wc_get_product($productId)) {
            return new WP_Error('not found', 'No product with the given id', ['status' => 404]);
        }

        echo json_encode($product->get_data());
    } catch (Exception $e) {
        throw new $e;
    }
    exit();
}

/**
 * Get categories list
 *
 * @param WP_REST_Request $request
 * @return mixed
 */
function apiGetCategoriesList(WP_REST_Request $request)
{
    try {
        $controller = new WC_REST_Product_Categories_Controller();
        $items = $controller->get_items($request);
        $categories = $items->get_data();

        if (!$categories) {
            return new WP_Error('not found', 'No categories found', ['status' => 404]);
        }

        echo json_encode($categories);
    } catch (Exception $e) {
        throw new $e;
    }
}

/**
 * Get top N products by category
 *
 * @param WP_REST_Request $request
 * @return mixed
 */
function apiGetTopProductsByCategory(WP_REST_Request $request)
{
    $params = $request->get_params();
    $categoryId = $params['id'];
    $topLimit = $params['limit'];

    try {
        $query = new WC_Product_Query([
            'limit' => $topLimit,
            'meta_key' => 'total_sales',
            'orderby' => 'meta_value_num',
            'order' => 'DESC',
            'return' => 'objects',
            'product_category' => [
                'id' => $categoryId,
            ],
        ]);

        $products = $query->get_products();

        if (!$products) {
            return new WP_Error('not found', 'No products with the given category', ['status' => 404]);
        }

        echo json_encode(
            array_map(function (WC_Product_Simple $product) {
                return $product->get_data();
            }, $products)
        );
    } catch (Exception $e) {
        throw new $e;
    }
    exit();
}

/**
 * Get all products by category
 *
 * @param WP_REST_Request $request
 * @return mixed
 */
function apiGetProductsByCategory(WP_REST_Request $request)
{
    $params = $request->get_params();
    $categoryId = $params['id'];

    try {
        $query = new WC_Product_Query([
            'limit' => $params['limit'] ?? get_option( 'posts_per_page' ),
            'page' => $params['page'] ?? 1,
            'offset' => $params['offset'] ?? '',
            'orderby' => 'name',
            'order' => 'ASC',
            'return' => 'objects',
            'product_category' => [
                'id' => $categoryId,
            ],
        ]);

        $products = $query->get_products();

        if (!$products) {
            return new WP_Error('not found', 'No products with the given category', ['status' => 404]);
        }

        echo json_encode(
            array_map(function (WC_Product_Simple $product) {
                return $product->get_data();
            }, $products)
        );
    } catch (Exception $e) {
        throw new $e;
    }
}

/**
 * Get products by vendor
 *
 * @param WP_REST_Request $request
 * @return mixed
 */
function apiGetProductsByVendor(WP_REST_Request $request)
{
    try {
        $vendorId = $request->get_param('id');
        /** @var WCMp_Vendor $vendor */
        if (!$vendor = get_wcmp_vendor($vendorId)) {
            return new WP_Error('not found', 'No vendor with the given id', ['status' => 404]);
        }
        $products = $vendor->get_products();

        if (!$products) {
            return new WP_Error('not found', 'No products with the given vendor id', ['status' => 404]);
        }

        $products = array_map(function (WP_Post $product) {
            return wc_get_product($product->ID)->get_data();
        }, $products);

        echo json_encode($products);
    } catch (Exception $e) {
        throw new $e;
    }
}

/**
 * Get order details
 *
 * @param WP_REST_Request $request
 * @return mixed
 */
function apiGetOrderDetails(WP_REST_Request $request)
{
    try {
        if(!is_user_logged_in()){
            return new WP_Error('unauthenticated', 'Unauthenticated', ['status' => 401]);
        }

        $user = wp_get_current_user();
        $orderId = $request->get_param('id');

        $orders = wc_get_orders([
            'customer_id' => $user->ID,
            'customer' => $user->user_email,
        ]);

        /** @var WC_Order $orderDetails */
        $orderDetails = null;
        foreach ($orders as $key => $order) {
            if ($order->get_id() === intval($orderId)) {
                $orderDetails = $order->get_data();
            }
        }

        if (!$orderDetails) {
            return new WP_Error('not found', 'No order with the given id', ['status' => 404]);
        }

        $itemsList = rearrangeItemsList($orderDetails['line_items']);
        $orderDetails['line_items'] = $itemsList;

        echo json_encode($orderDetails);
    } catch (Exception $e) {
        throw new $e;
    }
    exit();
}

function rearrangeItemsList(array $itemsList): array
{
    $items = array_map(function (WC_Order_Item_Product $item) {
        return $item->get_data();
    }, $itemsList);

    return $items;
}

/**
 * Get list of orders
 *
 * @return mixed
 */
function apiGetOrdersList()
{
    try {
        if(!is_user_logged_in()){
            return new WP_Error('unauthenticated', 'Unauthenticated', ['status' => 401]);
        }

        $user = wp_get_current_user();

        $orders = wc_get_orders([
            'customer_id' => $user->ID,
            'customer' => $user->user_email,
        ]);

        if (!$orders) {
            return new WP_Error('not found', 'No orders found', ['status' => 404]);
        }

        echo json_encode(
            array_map(function (WC_Order $order) {
                return $order->get_data();
            }, $orders)
        );
    } catch (Exception $e) {
        throw new $e;
    }
    exit();
}

/**
 * Get available payment methods
 *
 * @return mixed
 */
function apiGetPaymentMethods()
{
    try {
        $expayPaymentMethods = WC()->session->get('expay_methods');

        if (!$expayPaymentMethods) { //if there is no cached data
            $gatewaysData = WC()->payment_gateways()->get_available_payment_gateways();

            /** @var WC_Expay $gateways */
            $expayPaymentMethods = array_map(function ($gateway) {
                return $gateway->init_arr;
            }, $gatewaysData);
        }

        echo json_encode($expayPaymentMethods);
    } catch (Exception $e) {
        throw new $e;
    }

    exit();
}

/**
 * Get available shipping methods
 *
 * @param WP_REST_Request $request
 * @return mixed
 */
function apiGetShippingMethods(WP_REST_Request $request)
{
    try {
        if(!is_user_logged_in()){
            return new WP_Error('unauthenticated', 'Unauthenticated', ['status' => 401]);
        }

        $wc_controller = new WC_REST_Shipping_Methods_Controller();
        $items = $wc_controller->get_items($request);

        echo json_encode($items->get_data());
    } catch (Exception $e) {
        throw new $e;
    }
    exit();
}

/**
 * Make new order
 *
 * @param WP_REST_Request $request
 * @return mixed
 */
function apiMakeAnOrder(WP_REST_Request $request)
{
    try {
        if(!is_user_logged_in()){
            return new WP_Error('unauthenticated', 'Unauthenticated', ['status' => 401]);
        }

        $billing = $request->get_param('billing');
        $shipping = $request->get_param('shipping');

        /** @var WP_User $user */
        $user = wp_get_current_user();
        $product_id = $request->get_param('product_id');
        $product_quantity = $request->get_param('product_quantity');

        $addressTemplate = [
            'first_name',
            'last_name',
            'address_1',
            'address_2',
            'city',
            'state',
            'postcode',
            'country',
            'email',
            'phone',
            'company',
        ];

        $billingAddress = [];
        foreach ($billing as $param => $value) {
            if (in_array($param, $addressTemplate)) {
                $billingAddress[$param] = $value;
            } else {
                return new WP_Error('parameter not found', 'Wrong parameter name', ['status' => 400]);
            }
        }

        $emptyParameter = array_search('', $billingAddress);
        if ($emptyParameter) {
            if (!in_array($emptyParameter, ['address_2', 'company'])) {
                return new WP_Error('not found', 'Mandatory parameter is empty', ['status' => 400]);
            }
        }

        $shippingAddress = setOrderAddress($shipping);
        foreach ($shipping as $param => $value) {
            if (in_array($param, $addressTemplate)) {
                $shippingAddress[$param] = $value;
            } else {
                return new WP_Error('parameter not found', 'Wrong parameter name', ['status' => 400]);
            }
        }

        $emptyParameter = array_search('', $shippingAddress);
        if ($emptyParameter) {
            if (!in_array($emptyParameter, ['address_2', 'company'])) {
                return new WP_Error('not found', 'Mandatory parameter is empty', ['status' => 400]);
            }
        }

        foreach ([$product_id, $product_quantity, /*$paymentMethodId*/] as $value) {
            if (empty($value)) {
                return new WP_Error('parameter not found', 'Mandatory parameter is empty', ['status' => 400]);
            }
        }

        if (!$product = wc_get_product($product_id)) {
            return new WP_Error('not found', 'No product with the given id', ['status' => 400]);
        }

        $stockQuantity = $product->get_stock_quantity();
        $maxPurchaseQuantity = $product->get_max_purchase_quantity();

        if (($product_quantity <= 0 || $product_quantity >= $stockQuantity || $product_quantity >= $maxPurchaseQuantity)) {
            return new WP_Error('no products', 'Product quantity is exceeded', ['status' => 400]);
        }

        $order = wc_create_order(['customer_id' => $user->ID]);
        $order->set_address($billingAddress, 'billing');
        $order->set_address($shippingAddress, 'shipping');

        $order->add_product($product, $product_quantity);
        $order->calculate_totals();

        /** @see wc_get_order_statuses() */
        $order->update_status('wc-pending', 'Order received, no payment initiated', true);

        $orderDetails = $order->get_data();
        $itemsList = rearrangeItemsList($orderDetails['line_items']);
        $orderDetails['line_items'] = $itemsList;

        echo json_encode($orderDetails);
    } catch (Exception $e) {
        throw new $e;
    }
    exit();
}

/**
 * Initialize payment
 *
 * @param WP_REST_Request $request
 * @return mixed
 */
function apiProcessPayment(WP_REST_Request $request)
{
    try {
        if(!is_user_logged_in()){
            return new WP_Error('unauthenticated', 'Unauthenticated', ['status' => 401]);
        }
        //get orderId and paymentId along with payment method attributes
        $params = $request->get_params();

        $orderId = $params['orderId'];
        $paymentMethodId = $params['paymentId'];
        $attributes = $params['attr'];//payment required parameters

        $order = wc_get_order($orderId);
        $post_fields = false;

        $expayObject = new wc_expay();

        $expayRequest = new \expay\request_expay(
            $expayObject->get_option("mode"),
            $expayObject->get_option("merchant"),
            htmlspecialchars_decode($expayObject->get_option("secret_key"))
        );

        $gatewaysData = WC()->payment_gateways()->get_available_payment_gateways();

        foreach ($gatewaysData as $gateway) {
            /** @var WC_Expay $gateway */
            if ($gateway->init_arr['id'] === intval($paymentMethodId)) {
                $paymentMethod = $gateway;
                break;
            }
        }

        if (!$paymentMethod) {
            return new WP_Error('not found', 'No payment method for the given id', '400');
        }

        $order->set_payment_method($paymentMethod);

        //get required attributes to initialize payment
        $paymentAttributes = $paymentMethod->init_arr['attributes'];

        $paymentDataDynamic = [];
        foreach ($paymentAttributes as $key => $attribute) {
            foreach ($attribute as $parameter => $value) {
                if (($parameter === 'key')  && array_key_exists($value, $attributes)) {
                    $paymentDataDynamic[$value] = $attributes[$value];
                }
            }
        }

        /** @see wc_get_order_statuses() */
        $order->update_status('wc-pending', 'Order received, pending payment', true);

        if ($expayRequest) {
            $paymentDataDefault = [
                'service_id' => $paymentMethodId,
                'amount' => number_format($order->get_total(), 2, '.', ''),
                'order' => $order->get_order_number(),
                '_successUrl' => $order->get_view_order_url() . '?&status=success',
                '_waitingUrl' => $order->get_view_order_url() . '?&status=waiting',
                '_rejectUrl' => $order->get_view_order_url() . '?&status=reject',
            ];

            $paymentData = array_merge($paymentDataDefault, $paymentDataDynamic);

            try {
                $post_fields = $expayRequest->initPayment($paymentData);

            } catch (expay\ExpayException $e) {
            }
            if ($post_fields) {
                echo json_encode($post_fields);
            } else {
                echo json_encode($expayRequest->getinfoexpayerror());
            }
            exit();
        }

    } catch (Exception $e) {
        throw new $e;
    }
}
